require 'spec_helper'
require 'ostruct'
require 'pinpoint/formatter'

class Address < Struct.new(:name,
                           :street,
                           :locality,
                           :province,
                           :county,
                           :postal_code,
                           :country,
                           :latitude,
                           :longitude)
end

describe Pinpoint::Formatter do
  let(:address) do
    Address.new 'Kwik-E-Mart',
                '123 Apu Lane',
                'Springfield',
                'NW',
                'Springfield County',
                '12345',
                'United States',
                '12345',
                '67890'
  end

  describe '#format' do
    it 'can format it in one line US format' do
      expected = 'Kwik-E-Mart, 123 Apu Lane, Springfield, NW 12345, United States'

      expect(Pinpoint::Formatter.format(address, country: :us,
                                                 style:   :one_line_with_name)).
                                to eql(expected)
    end

    it 'can format it in multiline US format' do
      expected = <<-MULTILINE
Kwik-E-Mart
123 Apu Lane
Springfield, NW 12345
United States
      MULTILINE

      expect(Pinpoint::Formatter.format(address, country: :us,
                                                 style:   :multi_line_with_name)).
                                to eql(expected)
    end

    it 'can format it in HTML in US format' do
      expected = <<-MULTILINE
<address>
  <span class="section">
    <span class="name">Kwik-E-Mart</span>
  </span>
  <span class="section">
    <span class="street">123 Apu Lane</span>
  </span>
  <span class="section">
    <span class="locality">Springfield</span>
    <span class="province">NW</span>
    <span class="postal_code">12345</span>
  </span>
  <span class="section">
    <span class="country">United States</span>
  </span>
</address>
      MULTILINE

      expect(Pinpoint::Formatter.format(address, country: :us,
                                                 style:   :html)).
                                to eql(expected)
    end

    context 'when the address contains unsafe characters' do
      let(:address) do
        Address.new "<script>alert('Gotcha!');</script>",
                    '123 Apu Lane',
                    'Springfield',
                    'NW',
                    'Springfield County',
                    '12345',
                    'United States',
                    '12345',
                    '67890'
      end

      it 'escapes anything that is unsafe' do
        expected = <<-MULTILINE
<address>
  <span class="section">
    <span class="name">&lt;script&gt;alert(&#39;Gotcha!&#39;);&lt;/script&gt;</span>
  </span>
  <span class="section">
    <span class="street">123 Apu Lane</span>
  </span>
  <span class="section">
    <span class="locality">Springfield</span>
    <span class="province">NW</span>
    <span class="postal_code">12345</span>
  </span>
  <span class="section">
    <span class="country">United States</span>
  </span>
</address>
        MULTILINE

        expect(Pinpoint::Formatter.format(address, country: :us,
                                                   style:   :html)).
                                  to eql(expected)
      end
    end
  end
end
