require 'spec_helper'
require 'pinpoint/model_support'

class PinpointableAccessorClass
  def initialize(options = {})
    options.each do |key, value|
      send("#{key}=", value)
    end
  end
end

class PinpointableAccessorClassWithNoPrefixedFields < PinpointableAccessorClass
  attr_accessor :name,
                :street_and_premises,
                :city,
                :state,
                :county,
                :postal_code,
                :country,
                :latitude,
                :longitude
end

class PinpointableAccessorClassWithPrefixedFields < PinpointableAccessorClass
  attr_accessor :my_address_name,
                :my_address_street_and_premises,
                :my_address_city,
                :my_address_state,
                :my_address_county,
                :my_address_postal_code,
                :my_address_country,
                :my_address_latitude,
                :my_address_longitude
end

class PinpointableAccessorClassWithIncompleteFields < PinpointableAccessorClass
  attr_accessor :name,
                :street_and_premises,
                :city,
                :state
end

describe Pinpoint::ModelSupport do
  let(:model_support_module) { Pinpoint::ModelSupport }

  context 'when only a composed name is passed in' do
    let(:pinpointable_class) { PinpointableAccessorClassWithPrefixedFields }
    let(:pinpointable)       do
      pinpointable_class.new(
                                 my_address_name:                'name',
                                 my_address_street_and_premises: 'street',
                                 my_address_city:                'city',
                                 my_address_state:               'state',
                                 my_address_county:              'county',
                                 my_address_postal_code:         'postal_code',
                                 my_address_country:             'country',
                                 my_address_latitude:            'latitude',
                                 my_address_longitude:           'longitude')
    end

    it 'can add a reader to a class' do
      model_support_module.define_address_accessors(pinpointable_class,
                                                    field_name: :my_address)

      expect(pinpointable).to respond_to :my_address
    end

    it 'can compose an Address with the proper values from the model' do
      model_support_module.define_address_accessors(pinpointable_class,
                                                    field_name: :my_address)

      allow(Pinpoint::Address).to receive(:new)

      pinpointable.my_address

      expect(Pinpoint::Address).to have_received(:new).with(
        name:                'name',
        street_and_premises: 'street',
        city:                'city',
        state:               'state',
        county:              'county',
        postal_code:         'postal_code',
        country:             'country',
        latitude:            'latitude',
        longitude:           'longitude',
      )
    end

    it 'can deconstruct an Address into its component pieces' do
      model_support_module.define_address_accessors(pinpointable_class,
                                                    field_name: :my_address)

      address = Pinpoint::Address.new(name:                'name',
                                      street_and_premises: 'street',
                                      city:                'city',
                                      state:               'state',
                                      county:              'county',
                                      postal_code:         'postal_code',
                                      country:             'country',
                                      latitude:            'latitude',
                                      longitude:           'longitude')

      allow(pinpointable).to receive :my_address_name=
      allow(pinpointable).to receive :my_address_street_and_premises=
      allow(pinpointable).to receive :my_address_city=
      allow(pinpointable).to receive :my_address_state=
      allow(pinpointable).to receive :my_address_county=
      allow(pinpointable).to receive :my_address_postal_code=
      allow(pinpointable).to receive :my_address_country=
      allow(pinpointable).to receive :my_address_latitude=
      allow(pinpointable).to receive :my_address_longitude=

      pinpointable.my_address = address

      expect(pinpointable).to have_received(:my_address_name=).with('name')
      expect(pinpointable).to have_received(:my_address_street_and_premises=).with('street')
      expect(pinpointable).to have_received(:my_address_city=).with('city')
      expect(pinpointable).to have_received(:my_address_state=).with('state')
      expect(pinpointable).to have_received(:my_address_county=).with('county')
      expect(pinpointable).to have_received(:my_address_postal_code=).with('postal_code')
      expect(pinpointable).to have_received(:my_address_country=).with('country')
      expect(pinpointable).to have_received(:my_address_latitude=).with('latitude')
      expect(pinpointable).to have_received(:my_address_longitude=).with('longitude')
    end
  end

  context 'when a composed name is passed in along with a prefix' do
    let(:pinpointable_class) { PinpointableAccessorClassWithPrefixedFields }
    let(:pinpointable)       do
      pinpointable_class.new(my_address_name:                'name',
                             my_address_street_and_premises: 'street',
                             my_address_city:                'city',
                             my_address_state:               'state',
                             my_address_county:              'county',
                             my_address_postal_code:         'postal_code',
                             my_address_country:             'country',
                             my_address_latitude:            'latitude',
                             my_address_longitude:           'longitude')
    end

    it 'can add a reader to a class' do
      model_support_module.define_address_accessors(pinpointable_class,
                                                    field_name: :address,
                                                    prefix:     :my_address)

      expect(pinpointable).to respond_to :address
    end

    it 'can compose an Address with the proper values from the model' do
      model_support_module.define_address_accessors(pinpointable_class,
                                                    field_name: :address,
                                                    prefix:     :my_address)

      allow(Pinpoint::Address).to receive(:new).with(
        name:                'name',
        street_and_premises: 'street',
        city:                'city',
        state:               'state',
        county:              'county',
        postal_code:         'postal_code',
        country:             'country',
        latitude:            'latitude',
        longitude:           'longitude',
      )

      pinpointable.address
    end

    it 'can deconstruct an Address into its component pieces' do
      model_support_module.define_address_accessors(pinpointable_class,
                                                    field_name: :address,
                                                    prefix:     :my_address)

      address = Pinpoint::Address.new(name:                'name',
                                      street_and_premises: 'street',
                                      city:                'city',
                                      state:               'state',
                                      county:              'county',
                                      postal_code:         'postal_code',
                                      country:             'country',
                                      latitude:            'latitude',
                                      longitude:           'longitude')

      allow(pinpointable).to receive :my_address_name=
      allow(pinpointable).to receive :my_address_street_and_premises=
      allow(pinpointable).to receive :my_address_city=
      allow(pinpointable).to receive :my_address_state=
      allow(pinpointable).to receive :my_address_county=
      allow(pinpointable).to receive :my_address_postal_code=
      allow(pinpointable).to receive :my_address_country=
      allow(pinpointable).to receive :my_address_latitude=
      allow(pinpointable).to receive :my_address_longitude=

      pinpointable.address = address

      expect(pinpointable).to have_received(:my_address_name=).with('name')
      expect(pinpointable).to have_received(:my_address_street_and_premises=).with('street')
      expect(pinpointable).to have_received(:my_address_city=).with('city')
      expect(pinpointable).to have_received(:my_address_state=).with('state')
      expect(pinpointable).to have_received(:my_address_county=).with('county')
      expect(pinpointable).to have_received(:my_address_postal_code=).with('postal_code')
      expect(pinpointable).to have_received(:my_address_country=).with('country')
      expect(pinpointable).to have_received(:my_address_latitude=).with('latitude')
      expect(pinpointable).to have_received(:my_address_longitude=).with('longitude')
    end
  end

  context 'when an object is referenced which does not implement all of the fields' do
    let(:pinpointable_class) { PinpointableAccessorClassWithIncompleteFields }
    let(:pinpointable)       do
      pinpointable_class.new(name:                'name',
                             street_and_premises: 'street',
                             city:                'city',
                             state:               'state')
    end

    it 'can compose an Address with the proper values from the model' do
      model_support_module.define_address_accessors(pinpointable_class,
                                                    field_name: :address,
                                                    prefix:     false)

      allow(Pinpoint::Address).to receive(:new).with(
        name:                'name',
        street_and_premises: 'street',
        city:                'city',
        state:               'state',
      )

      pinpointable.address
    end

    it 'can deconstruct an Address into its component pieces' do
      model_support_module.define_address_accessors(pinpointable_class,
                                                    field_name: :address,
                                                    prefix:     false)

      address = Pinpoint::Address.new(name:                'name',
                                      street_and_premises: 'street',
                                      city:                'city',
                                      state:               'state',
                                      county:              'county',
                                      postal_code:         'postal_code',
                                      country:             'country',
                                      latitude:            'latitude',
                                      longitude:           'longitude')

      allow(pinpointable).to receive :name=
      allow(pinpointable).to receive :street_and_premises=
      allow(pinpointable).to receive :city=
      allow(pinpointable).to receive :state=

      pinpointable.address = address

      expect(pinpointable).to have_received(:name=).with('name')
      expect(pinpointable).to have_received(:street_and_premises=).with('street')
      expect(pinpointable).to have_received(:city=).with('city')
      expect(pinpointable).to have_received(:state=).with('state')
    end
  end

  context 'when a composed name is not passed in' do
    it 'throws an error' do
      expect { model_support_module.define_address_accessors(pinpointable_class) }.to(
        raise_error)
    end
  end
end
