require 'spec_helper'

class ValidatablePinpointable
  include ActiveModel::Validations
  extend Pinpoint::Composable

  attr_accessor :name,
                :street_and_premises,
                :city,
                :state,
                :postal_code,
                :country

  pinpoint :address, prefix:   false,
                     validate: true
end

class UnvalidatablePinpointable
  include ActiveModel::Validations
  extend Pinpoint::Composable

  attr_accessor :address_name,
                :address_street_and_premises,
                :address_city,
                :address_state,
                :address_postal_code,
                :address_country

  pinpoint :address, prefix: :address
end

describe 'Pinpoint::Validations' do
  context 'when the pinpoint object has validations enabled' do
    subject { ValidatablePinpointable.new }

    describe '#address_name' do
      it { expect(subject).to     allow_value(nil, ('*' * 140)).for(:name) }
      it { expect(subject).not_to allow_value(('*' * 141)).for(:name) }
    end

    describe '#address_state' do
      it { expect(subject).to     allow_value(nil, ('*' * 255)).for(:street_and_premises) }
      it { expect(subject).not_to allow_value(('*' * 256)).for(:street_and_premises) }
    end

    describe '#address_city' do
      it { expect(subject).to     allow_value(nil, ('*' * 60)).for(:city) }
      it { expect(subject).not_to allow_value(('*' * 61)).for(:city) }
    end

    describe '#address_street_and_premises' do
      it { expect(subject).to     allow_value(nil, '', 'NY', 'MO').for(:state) }
      it { expect(subject).not_to allow_value('WP', 'NI', 'PO').for(:state) }
    end

    describe '#address_postal_code' do
      it { expect(subject).to     allow_value(nil, '', 12_345, 12_345 - 1234).for(:postal_code) }
      it { expect(subject).not_to allow_value('asdf', '123456').for(:postal_code) }
    end

    context 'when the street address is set' do
      before { subject.street_and_premises = 'foo' }

      it('the city should not be valid when blank')     { expect(subject).not_to allow_value(nil, '').for(:city) }
      it('the state should not be valid when blank')    { expect(subject).not_to allow_value(nil, '').for(:state) }
      it('the zip code should not be valid when blank') { expect(subject).not_to allow_value(nil, '').for(:postal_code) }
    end

    context 'when the city is set' do
      before { subject.city = 'foo' }

      it('the street should not be valid when blank')   { expect(subject).not_to allow_value(nil, '').for(:street_and_premises) }
      it('the state should not be valid when blank')    { expect(subject).not_to allow_value(nil, '').for(:state) }
      it('the zip code should not be valid when blank') { expect(subject).not_to allow_value(nil, '').for(:postal_code) }
    end

    context 'when the state is set' do
      before { subject.state = 'FO' }

      it('the street should not be valid when blank')   { expect(subject).not_to allow_value(nil, '').for(:street_and_premises) }
      it('the city should not be valid when blank')     { expect(subject).not_to allow_value(nil, '').for(:city) }
      it('the zip code should not be valid when blank') { expect(subject).not_to allow_value(nil, '').for(:postal_code) }
    end

    context 'when the zip is set' do
      before { subject.postal_code = '12345' }

      it('the street should not be valid when blank')   { expect(subject).not_to allow_value(nil, '').for(:street_and_premises) }
      it('the city should not be valid when blank')     { expect(subject).not_to allow_value(nil, '').for(:city) }
      it('the state should not be valid when blank')    { expect(subject).not_to allow_value(nil, '').for(:state) }
    end
  end

  context 'when the pinpoint object has validations enabled' do
    subject { UnvalidatablePinpointable.new }

    describe '#address_name' do
      it { expect(subject).to     allow_value(('*' * 141)).for(:address_name) }
    end

    describe '#address_state' do
      it { expect(subject).to     allow_value(('*' * 256)).for(:address_street_and_premises) }
    end

    describe '#address_city' do
      it { expect(subject).to     allow_value(('*' * 61)).for(:address_city) }
    end

    describe '#address_street_and_premises' do
      it { expect(subject).to     allow_value('WP', 'NI', 'PO').for(:address_state) }
    end

    describe '#address_postal_code' do
      it { expect(subject).to     allow_value('asdf', '123456').for(:address_postal_code) }
    end

    context 'when the street address is set' do
      before { subject.address_street_and_premises = 'foo' }

      it('the city should not be valid when blank')     { expect(subject).to allow_value(nil, '').for(:address_city) }
      it('the state should not be valid when blank')    { expect(subject).to allow_value(nil, '').for(:address_state) }
      it('the zip code should not be valid when blank') { expect(subject).to allow_value(nil, '').for(:address_postal_code) }
    end

    context 'when the city is set' do
      before { subject.address_city = 'foo' }

      it('the street should not be valid when blank')   { expect(subject).to allow_value(nil, '').for(:address_street_and_premises) }
      it('the state should not be valid when blank')    { expect(subject).to allow_value(nil, '').for(:address_state) }
      it('the zip code should not be valid when blank') { expect(subject).to allow_value(nil, '').for(:address_postal_code) }
    end

    context 'when the state is set' do
      before { subject.address_state = 'FO' }

      it('the street should not be valid when blank')   { expect(subject).to allow_value(nil, '').for(:address_street_and_premises) }
      it('the city should not be valid when blank')     { expect(subject).to allow_value(nil, '').for(:address_city) }
      it('the zip code should not be valid when blank') { expect(subject).to allow_value(nil, '').for(:address_postal_code) }
    end

    context 'when the zip is set' do
      before { subject.address_postal_code = '12345' }

      it('the street should not be valid when blank')   { expect(subject).to allow_value(nil, '').for(:address_street_and_premises) }
      it('the city should not be valid when blank')     { expect(subject).to allow_value(nil, '').for(:address_city) }
      it('the state should not be valid when blank')    { expect(subject).to allow_value(nil, '').for(:address_state) }
    end
  end
end
